library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity calculator is
	port(
		binary : in std_logic_vector(3 downto 0);	-- binary rep of input
		isnum : in std_logic;			-- whether the input is a number
		rst : in std_logic;				-- reset signal
		clk : in std_logic;			-- clock signal
		valid: in std_logic;			-- valid signal (from kb_to_binary module)
		result : out std_logic_vector(3 downto 0)	-- binary rep of output
	);
end calculator;

architecture c_arch of calculator is
	constant firstdig : std_logic_vector(2 downto 0) := "000";
	constant seconddig : std_logic_vector(2 downto 0) := "001";
	constant op : std_logic_vector(2 downto 0) := "010";
	
	signal currentValue : integer range 0 to 255;
	signal currentMode : std_logic_vector(3 downto 0);
	signal state : std_logic_vector(2 downto 0);

	function eval (val1, val2: integer; op1 : std_logic_vector(3 downto 0)) return integer is
	begin
		case op1 is
			when X"0" =>	-- no op
				return val2;
			when X"1" => 	-- /	-- because Foundation will not implement division
						-- by a variable, we will not be using it
--				if not (val2 = 0) then
--					return val1 / val2;
--				else
					return val1;
--				end if;
			when X"2" => 	-- *
				return val1 * val2;
			when X"4" => 	-- +
				return val1 + val2;
			when X"3" =>	-- -
				if val1 > val2 then
					return val1 - val2;
				else
					return 0;
				end if;
			when others =>
				return 0;
		end case;
	end eval;
	
begin

	result <= std_logic_vector(to_signed(currentValue, 4));
	
	process(rst, clk)
	variable first: std_logic_vector(3 downto 0);
	variable operation : std_logic_vector(3 downto 0) ;
	variable currentDigit, prev : integer range 0 to 255;
	begin
	
		if rst = '1' then
			currentValue <= 0;
			state <= firstdig;
			operation := X"0";
			first := x"0";
			currentDigit := 0;
			prev := 0;
		elsif clk'event and clk = '1' then
		    if valid = '1' then
			case state is
				when firstdig =>
					if isnum = '0' then
						case binary is
							when X"6" => --clear
								currentValue <= 0;
								operation := X"0";
								currentDigit := 0;
								prev := 0;
							when X"5" => -- enter
								null;
							when X"A" => -- backspace
								currentValue <= 0;
							when others =>
								null;
						end case; 
					elsif isnum = '1' then
						prev := currentValue;
						currentDigit := to_integer(unsigned(binary));
						currentValue <= to_integer(unsigned(binary));
						state <= op;
					end if;
				when op => 
					if isnum = '0' then
						case binary is
							when X"6" => 	-- clear
								currentValue <= 0;
								state <= firstdig;
								operation := X"0";
								currentDigit := 0;
							when X"5" =>	-- enter
								currentValue <= eval(prev, currentDigit, operation);
								operation := X"0";
								state <= firstdig;
							when X"1" | X"2" | X"3" | X"4" =>	-- operation
								currentValue <= eval(prev, currentDigit, operation);
								operation := binary;
								state <= firstdig;
							when X"A" =>
								state <= firstdig;
								currentValue <= 0;
								currentDigit := 0;
							when others => null;
						end case;
					end if;
				when others => null;
			end case;
		    end if;
		end if;
	end process;
end c_arch;





