library IEEE;
use IEEE.std_logic_1164.all;

entity kb_to_binary is
    port (
        sc: in STD_LOGIC_VECTOR (7 downto 0);			-- input scancode
		bin: out STD_LOGIC_VECTOR (3 downto 0);			-- binary representation
		isnum: out STD_LOGIC;							-- whether input is a number
		kb_prs: in STD_LOGIC;							-- keyboard press signal
		clk: in STD_LOGIC;								-- clock
		valid: out STD_LOGIC							-- whether input is valid
    );
end kb_to_binary;

architecture kb_to_binary_arch of kb_to_binary is
begin

  -----------------------------------------------------------------------------
  -- TranslationProcess: converts the given scancodes into binary that
  --                     the calculator unit will handle
  -----------------------------------------------------------------------------
  TranslationProcess:process(clk, kb_prs, sc)
  begin
      if (kb_prs'event and kb_prs = '1') then
        if (sc = X"70") or (sc = X"45") then    -- 0
          bin <= X"0";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"69") or (sc = X"16") then -- 1
          bin <= X"1";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"72") or (sc = X"1E") then -- 2
          bin <= X"2";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"7A") or (sc = X"26") then -- 3
          bin <= X"3";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"6B") or (sc = X"25") then -- 4
          bin <= X"4";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"73") or (sc = X"2E") then -- 5
          bin <= X"5";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"74") or (sc = X"36") then -- 6
          bin <= X"6";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"6C") or (sc = X"3D") then -- 7
          bin <= X"7";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"75") or (sc = X"3E") then -- 8
          bin <= X"8";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"7D") or (sc = X"46") then -- 9
          bin <= X"9";
          isnum <= '1';
          valid <= '1';
        elsif (sc = X"1C") then -- A
          isnum <= '1';
          valid <= '1';
          bin <= X"A";
        elsif (sc = X"32") then -- B
          isnum <= '1';
          valid <= '1';
          bin <= X"B";
        elsif (sc = X"21") then -- C
          isnum <= '1';
          valid <= '1';
          bin <= X"C";
        elsif (sc = X"23") then -- D
          isnum <= '1';
          valid <= '1';
          bin <= X"D";
        elsif (sc = X"24") then -- E
          isnum <= '1';
          valid <= '1';
          bin <= X"E";
        elsif (sc = X"2B") then -- F                
          isnum <= '1';
          valid <= '1';
          bin <= X"F";
        elsif (sc = X"4A") then -- /
          bin <= X"1";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"7C") then -- *
          bin <= X"2";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"7B") then -- -
          bin <= X"3";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"79") then -- +
          bin <= X"4";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"29") then -- spacebar
          bin <= X"6";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"66") then -- bksp
          bin <= X"A";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"4D") then -- P
          bin <= X"7";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"44") then -- O
          bin <= X"8";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"43") then -- I
          bin <= X"9";
          isnum <= '0';
          valid <= '1';
        elsif (sc = X"5A") then -- enter
          bin <= X"5";
          isnum <= '0';
          valid <= '1';  
        else  -- invalid, ignore it
          bin <= X"0";
          isnum <= '0';
          valid <= '0';
        end if;
      end if;
  end process;
 
end kb_to_binary_arch;


